<?php

declare(strict_types = 1);

namespace rark\simple_fill;

use pocketmine\plugin\PluginBase;
use pocketmine\item\Item;
use rark\simple_fill\listener\{PlayerEventListener, BlockEventListener};
use rark\simple_fill\command\{FillCommand, UndoCommand};
use rark\simple_fill\utils\{Fill, Undo};


final class Main extends PluginBase{
	/** @var string */
	public const HEADER = '[SimpleFill]';
	/** @var self */
	public static $instance;
	/** @var Item */
	public static $item;
	/** @var Fill */
	public static $fill;
	/** @var Undo */
	public static $undo;

	public function onEnable(){
		$this->setObject();
		$this->registerListener();
		$this->registerCommand();
		$this->loadFile();
	}

	private function setObject():void{
		self::$instance = $this;
		self::$fill = new Fill;
		self::$undo = new Undo;
		$this->setItem();
	}

	private function setItem():void{
		$item = Item::get(450);
		$item->setCustomName('§aSwitchFillMode');
		$item->setLore([
			'タップでON/OFF切り替え',
			'ON状態の時にブロックを',
			'二か所に設置で設置でFill'
		]);
		self::$item = $item;
	}

	private function registerListener():void{
		$listeners = [
			new PlayerEventListener,
			new BlockEventListener
		];
		
		foreach($listeners as $listener){
			$this->getServer()->getPluginManager()->registerEvents($listener, $this);
		}
	}

	private function registerCommand():void{
		$this->getServer()->getCommandMap()->registerAll('[Rark]SimpleFill', [
			new FillCommand($this),
			new UndoCommand($this)
		]);
	}

	private function loadFile():void{
		require_once($this->getServer()->getPluginPath().$this->getName().'/src/rark/simple_fill/utils/sound.php');
		require_once($this->getServer()->getPluginPath().$this->getName().'/src/rark/simple_fill/utils/sortPos.php');
		require_once($this->getServer()->getPluginPath().$this->getName().'/src/rark/simple_fill/utils/in_region.php');
	}
}